@@ -20,6 +20,10 @@ Implements limitedinteraction functions.
 
 This file is not to be included as a module but instead called as a separate
 process by limitedinteraction/__init__.py.
+
+Functions that return something print a json string with [returnval, contents] where
+returnval is '' for success or a string that represent an exception to raise (e.g.,
+'ModuleNotFoundError') by the module on error.
 """
 
 __author__ = "Félix Chénier"
@@ -28,15 +32,41 @@ __email__ = "chenier.felix@uqam.ca"
 __license__ = "Apache 2.0"
 
 
-# Imports
-import tkinter as tk
-import tkinter.ttk as ttk
-import tkinter.filedialog as filedialog
+
+#---- Main imports
+import json
+import sys
+
+
+#---- Exception management
+def exit_and_raise(exception_type: str, exception_text: str):
+    """Print [exception_type, exception_text] as a json string and exit."""
+    print(json.dumps([exception_type, exception_text]))
+    sys.exit(0)
+
+
+#---- Other imports
+
+# Try to import tkinter and gracefully fail if not available
+try:
+    import tkinter as tk
+    import tkinter.ttk as ttk
+    import tkinter.filedialog as filedialog
+except ModuleNotFoundError:
+    exit_and_raise(
+        'ModuleNotFoundError', (
+            "Failed to import tkinter. \n"
+            "If you are seeing this message and are using your operating system's \n"
+            "supplied python, you may try to install the tk GUI toolkit, or best, \n"
+            "install a proper version of python (e.g., python.org, anaconda.org, \n"
+            "etc.) which normally includes tkinter and links correctly to the tk \n"
+            "GUI toolkit."))
+
+
 from functools import partial
 import time
 import os
 import platform
-import json
 import sys
 
 
@@ -65,22 +95,51 @@ def get_window_geometry(root):
 
 def place_window(root, **kwargs):
     """Place window in screen."""
-    (width, height, left, top) = get_window_geometry(root)
+    if 'left' in kwargs and 'right' in kwargs:
+        exit_and_raise('ValueError',
+                       "'left' and 'right' cannot be both specified.")
+    if 'top' in kwargs and 'bottom' in kwargs:
+        exit_and_raise('ValueError',
+                       "'top' and 'bottom' cannot be both specified.")
+
+    # Get current dimensions
+    (contents_width, contents_height,
+     contents_left, contents_top) = get_window_geometry(root)
+
+    # Calculate frame width and titlebar height
+    # From https://exceptionshub.com/
+    #     how-to-center-a-window-on-the-screen-in-tkinter.html
+    frm_width = root.winfo_rootx() - root.winfo_x()
+    win_width = contents_width + 2 * frm_width
+    titlebar_height = root.winfo_rooty() - root.winfo_y()
+    win_height = contents_height + titlebar_height + frm_width
+
+    # Ensure to meet given minimal values for width and height
+    if win_width < kwargs['min_width']:
+        win_width = kwargs['min_width']
+
+    if 'min_height' in kwargs and win_height < kwargs['min_height']:
+        win_height = kwargs['min_height']
 
-    if width < kwargs['min_width']:
-        width = kwargs['min_width']
-    if width < kwargs['min_height']:
-        height = kwargs['min_height']
     if 'left' in kwargs:
-        left = kwargs['left']
+        win_left = kwargs['left']
+    elif 'right' in kwargs:
+        win_left = root.winfo_screenwidth() - win_width - kwargs['right']
     else:
-        left = int(root.winfo_screenwidth() / 2 - width / 2)  # center
-    if 'top' in kwargs:
-        top = kwargs['top']
-    else:
-        top = int(root.winfo_screenheight() / 2 - height / 2)  # center
+        win_left = int(root.winfo_screenwidth() / 2 - win_width / 2)  # center
 
-    root.geometry(f'{width}x{height}+{left}+{top}')
+    if 'top' in kwargs:
+        win_top = kwargs['top']
+    elif 'bottom' in kwargs:
+        win_top = root.winfo_screenheight() - win_height - kwargs['bottom']
+    else:
+        win_top = int(root.winfo_screenheight() / 2 - win_height / 2)  # center
+
+    # Window parameters to contents paramaters
+    contents_width = win_width - 2 * frm_width
+    contents_height = win_height - titlebar_height - frm_width
+
+    root.geometry(f'{contents_width}x{contents_height}+{win_left}+{win_top}')
 
 
 def show_window(root):
@@ -100,14 +159,18 @@ def button_dialog(root, frame, **kwargs):
         root.quit()
 
     # Buttons
+    buttons = []
     ichoice = 0
     for choice in kwargs['choices']:
         btn = ttk.Button(frame,
-                        text=choice,
-                        command=partial(return_choice, ichoice))
+                         text=choice,
+                         command=partial(return_choice, ichoice))
         btn.pack(fill=tk.X)
+        buttons.append(btn)
         ichoice = ichoice + 1
 
+    buttons[0].focus()
+
     place_window(root, **kwargs)
     show_window(root)
     root.mainloop()
@@ -117,10 +180,10 @@ def button_dialog(root, frame, **kwargs):
 def input_dialog(root, frame, **kwargs):
     """Terminate composing the GUI and run it."""
     # Condition inputs
-    if 'descriptions' in kwargs:
-        descriptions = kwargs['descriptions']
+    if 'labels' in kwargs:
+        labels = kwargs['labels']
     else:
-        descriptions = []
+        labels = []
 
     if 'initial_values' in kwargs:
         initial_values = kwargs['initial_values']
@@ -133,59 +196,62 @@ def input_dialog(root, frame, **kwargs):
         masked = []
 
     n_boxes = max(1,
-                  len(descriptions),
+                  len(labels),
                   len(initial_values),
                   len(masked))
 
-    if len(descriptions) == 0:
-        descriptions = [''] * n_boxes
+    if len(labels) == 0:
+        labels = [''] * n_boxes
     if len(initial_values) == 0:
         initial_values = [''] * n_boxes
     if len(masked) == 0:
         masked = [False] * n_boxes
 
     if (
-            len(descriptions) != n_boxes or
+            len(labels) != n_boxes or
             len(initial_values) != n_boxes or
             len(masked) != n_boxes):
-        return ("!!!ERROR!!! Length mismatch between descriptions, "
-                "initial_values and masked.")
+        exit_and_raise('ValueError', ("Length mismatch between labels, "
+                                      "initial_values and masked."))
 
     outputs = [[]]
 
     # OK callback
     def ok_pressed(*args):
-        for entry in entries:
-            outputs[0].append(entry.get())
+        for tk_entry in tk_entries:
+            outputs[0].append(tk_entry.get())
         root.quit()
 
     # Add labels and entries
-    labels = []
-    entries = []
-    for i, description in enumerate(descriptions):
+    tk_labels = []
+    tk_entries = []
+    for i, label in enumerate(labels):
 
         # Label
-        if len(description) > 0:
-            label = ttk.Label(frame, text=description)
-            label.pack(fill=tk.X)
-            label.configure(anchor="center")  # center justified
-            labels.append(label)
+        if len(label) > 0:
+            tk_label = ttk.Label(frame, text=label)
+            tk_label.pack(fill=tk.X)
+            tk_label.configure(anchor="center")  # center justified
+            tk_labels.append(tk_label)
 
         # Entry
         if masked[i]:
-            entry = ttk.Entry(frame, show='*')
+            tk_entry = ttk.Entry(frame, show='*')
         else:
-            entry = ttk.Entry(frame)
+            tk_entry = ttk.Entry(frame)
 
-        entry.insert(0, initial_values[i])
-        entry.bind('<Return>', ok_pressed)
-        entry.pack(fill=tk.X)
-        entries.append(entry)
+        tk_entry.insert(0, initial_values[i])
+        tk_entry.bind('<Return>', ok_pressed)
+        tk_entry.pack(fill=tk.X)
+        if i == 0:
+            tk_entry.focus()
+        tk_entries.append(tk_entry)
 
     # Add OK button
-    ok_btn = ttk.Button(frame, text='OK', command=ok_pressed)
-    ok_btn.bind('<Return>', ok_pressed)
-    ok_btn.pack(fill=tk.X)
+    tk_ok_btn = ttk.Button(frame, text='OK', command=ok_pressed,
+                           default='active')
+    tk_ok_btn.bind('<Return>', ok_pressed)
+    tk_ok_btn.pack(fill=tk.X)
 
     place_window(root, **kwargs)
     show_window(root)
@@ -254,8 +320,6 @@ if __name__ == '__main__':
         kwargs['message'] = ''
     if 'min_width' not in kwargs:
         kwargs['min_width'] = 100
-    if 'min_height' not in kwargs:
-        kwargs['min_height'] = 100
 
     root = tk.Tk()
 
@@ -313,16 +377,18 @@ if __name__ == '__main__':
     lbl.configure(anchor="center")  # center justified
     lbl.pack(fill=tk.X)
 
-    #--- PASS THE REST TO THE REQUESTED FUNCTION ---#
+    #---- Pass the rest to the requested function and return
 
-    if function == 'button_dialog':
-        print(json.dumps(button_dialog(root, frame, **kwargs)))
+    if function == 'import':
+        print(json.dumps(['', '']))
+    elif function == 'button_dialog':
+        print(json.dumps(['', button_dialog(root, frame, **kwargs)]))
     elif function == 'input_dialog':
-        print(json.dumps(input_dialog(root, frame, **kwargs)))
+        print(json.dumps(['', input_dialog(root, frame, **kwargs)]))
     elif function == 'message':
         message(root, frame, **kwargs)
     elif function == 'get_filename':
-        print(json.dumps(get_filename(root, **kwargs)))
+        print(json.dumps(['', get_filename(root, **kwargs)]))
     elif function == 'get_folder':
-        print(json.dumps(get_folder(root, **kwargs)))
+        print(json.dumps(['', get_folder(root, **kwargs)]))
 
